/*
 * mlip.c
 * Monolith's DynDNS update client
 * Copyright (C) October, 1998 Sergei Viznyuk <sv@phystech.com>
 * this code has been derived from a code by
 * Artur Skawina <skawina@usa.net>
 * http://www.ml.org/ml/dyndns/clients.html
 * 
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <syslog.h>
#include <errno.h>
#include "mlip.h"

char *ProgramName;
int DebugFlag,IsTTY;
struct hostent	*hp;
void signalSetup();
unsigned benc(unsigned char *dst,unsigned char *src,unsigned slen);
/*****************************************************************************/
int tcpConnectByName(ServerName,ServerPort)
char *ServerName;
unsigned short ServerPort;
{
  struct sockaddr_in saddr;
  int s;

  if ( (hp=gethostbyname(ServerName)) == NULL )
    {
      if ( IsTTY )
	fprintf(stderr,"gethostbyname(\"%s\"): %s\n",
		ServerName,sys_errlist[errno]);
      else
        syslog(LOG_ERR,"gethostbyname(\"%s\"): %m\n",ServerName);
      return -1;
    }
  saddr.sin_family = hp->h_addrtype;
  memcpy((char *)&saddr.sin_addr,hp->h_addr,sizeof(saddr.sin_addr));
  saddr.sin_port=htons(ServerPort);
  if ( (s=socket(AF_INET,SOCK_STREAM,0)) == -1 )
    {
      if ( IsTTY )
	fprintf(stderr,"socket(..): %s\n",sys_errlist[errno]);
      else
        syslog(LOG_ERR,"socket(..): %m\n");
      return -1;
    }
  if ( DebugFlag && IsTTY )
    fprintf(stderr,"%s: connecting to "URLHOST" (%u.%u.%u.%u)\n",ProgramName,
    ((unsigned char *)&saddr.sin_addr)[0],
    ((unsigned char *)&saddr.sin_addr)[1],
    ((unsigned char *)&saddr.sin_addr)[2],
    ((unsigned char *)&saddr.sin_addr)[3]);
  if ( connect(s,(struct sockaddr *)&saddr,sizeof(saddr)) == -1 )
    {
      if ( IsTTY )
	fprintf(stderr,"connect(%d,..): %s\n",s,sys_errlist[errno]);
      else
        syslog(LOG_ERR,"connect(%d,..): %m\n",s);
      close(s);
      return -1;
    }
  if ( DebugFlag )
    if ( IsTTY )
      fprintf(stderr,"%s: tcp/ip connected to %s (%u.%u.%u.%u)\n",
      ProgramName,ServerName,((unsigned char *)&saddr.sin_addr.s_addr)[0],
      ((unsigned char *)&saddr.sin_addr.s_addr)[1],
      ((unsigned char *)&saddr.sin_addr.s_addr)[2],
      ((unsigned char *)&saddr.sin_addr.s_addr)[3]);
    else
      syslog(LOG_DEBUG,"tcp/ip connected to %s (%u.%u.%u.%u)\n",
      ServerName,((unsigned char *)&saddr.sin_addr.s_addr)[0],
      ((unsigned char *)&saddr.sin_addr.s_addr)[1],
      ((unsigned char *)&saddr.sin_addr.s_addr)[2],
      ((unsigned char *)&saddr.sin_addr.s_addr)[3]);
  return s;
}
/*****************************************************************************/
int main(argn,argc)
int argn;
char *argc[];
{
  struct sockaddr_in saddr;
  char upw[128],buf[READ_BUF_SIZE],ipaddr[16];
  int slen=sizeof(saddr);
  int i=1;
  int k=1;
  int s=1;

  IsTTY=isatty(2);
  if ( ! IsTTY )
    openlog(PROGRAM_NAME,LOG_PID,LOG_LOCAL0);

  while ( argc[i] )
    if ( argc[i][0]=='-' )
prgs: switch ( argc[i][s] )
	{
	  case 0:
	    i++;
	    s=1;
	    break;
	  case 'd':
	    s++;
	    DebugFlag=1;
	    goto prgs;
          default:
usage:	    if ( IsTTY ) fprintf(stderr,"\
MLIP Monolith\'s Update Client v."PROGRAM_VERSION"\n\
Copyright (C) October, 1998 Sergei Viznyuk <sv@phystech.com>\n\
Usage: mlip [-d] [ipaddress]\n");
	    exit(1);
	}
    else
      argc[k++]=argc[i++];
  if ( k > 2 ) goto usage;
  ProgramName=argc[0];
  signalSetup();
  alarm(TIME_OUT);
  s=tcpConnectByName(URLHOST,URLPORT);
  if ( s == -1 ) exit(1);
  if ( k == 1 )
    {
      if ( getsockname(s,(struct sockaddr *)&saddr,&slen) == -1 )
        {
          if ( IsTTY )
	    fprintf(stderr,"%s: getsockname(%d,..): %s\n",
	    ProgramName,s,sys_errlist[errno]);
	  else
	    syslog(LOG_ERR,"getsockname(%d,..): %m\n",s);
	  goto err;
        }
      sprintf(ipaddr,"%u.%u.%u.%u",
      ((unsigned char *)&saddr.sin_addr.s_addr)[0],
      ((unsigned char *)&saddr.sin_addr.s_addr)[1],
      ((unsigned char *)&saddr.sin_addr.s_addr)[2],
      ((unsigned char *)&saddr.sin_addr.s_addr)[3]);
      if ( DebugFlag && IsTTY )
	fprintf(stderr,"%s: using current ipaddr=%s\n",ProgramName,ipaddr);
    }
  else
    snprintf(ipaddr,16,"%s",argc[1]);
  benc(upw,ML_USERNAME_PASSWORD,strlen(ML_USERNAME_PASSWORD));
  i=snprintf(buf,READ_BUF_SIZE,"GET /mis-bin/ms3/nic/dyndns?"
              "command=Update+Host&"
              "do=mod&"
              "domain="ML_HOSTNAME"&"
              "act=act&"
              "wildcard=on&"
	      "ipaddr=%s&"
              "agree=agree HTTP/1.0\r\n"
              "Pragma: no-cache\r\n"
              "User-Agent: "PROGRAM_NAME"/"PROGRAM_VERSION"\r\n"
              "Authorization: Basic %s\r\n"
              "\r\n",ipaddr,upw);
  if ( write(s,buf,i) == -1 )
    {
      if ( IsTTY )
        fprintf(stderr,"%s: write(%d,..): %s\n",ProgramName,s,
	sys_errlist[errno]);
      else
        syslog(LOG_ERR,"write(%d,..): %m\n",s);
      goto err;
    }
  k=0;
  while ( (i=read(s,buf+k,READ_BUF_SIZE-k)) > 0 ) k+=i;
  buf[k]=0;
  if ( k )
    {
/*      i=0; while ( buf[i] < 32 ) i++; while ( buf[i] > 31 ) i++; buf[i]=0; */
      if ( strstr(buf,ML_OKSTRING) )
        if ( IsTTY )
	  fprintf(stderr,"%s: DNS entry for "ML_HOSTNAME" updated with %s\n",
	  ProgramName,ipaddr);
        else
          syslog(LOG_INFO,"DNS entry for "ML_HOSTNAME" updated with %s\n",
	  ipaddr);
      else
        {
          if ( IsTTY )
            fprintf(stderr,"%s: \"%s\" response received from "URLHOST"\n",
	    ProgramName,buf);
          else
            syslog(LOG_ERR,"\"%s\" response received from "URLHOST"\n",buf);
	  goto err;
        }
      close(s);
      exit(0);
    }
err:
  close(s);
  if ( IsTTY )
    fprintf(stderr,"%s: error updating DNS entry for "ML_HOSTNAME" with %s\n",
    ProgramName,ipaddr);
  else
    syslog(LOG_ERR,"error updating DNS entry for "ML_HOSTNAME" with %s\n",ipaddr);
  exit(errno);
}
